import { ClickEvent } from '@nebula.gl/edit-modes';
import { CSSProperties } from 'react';
import { DraggingEvent } from '@nebula.gl/edit-modes';
import { DrawCircleByDiameterMode } from '@nebula.gl/edit-modes';
import { DrawCircleFromCenterMode } from '@nebula.gl/edit-modes';
import { DrawLineStringMode } from '@nebula.gl/edit-modes';
import { DrawPointMode } from '@nebula.gl/edit-modes';
import { DrawPolygonByDraggingMode } from '@nebula.gl/edit-modes';
import { DrawPolygonMode } from '@nebula.gl/edit-modes';
import { DrawRectangleMode } from '@nebula.gl/edit-modes';
import type { EditAction } from '@nebula.gl/edit-modes';
import { EditMode } from '@nebula.gl/edit-modes';
import { Feature } from '@nebula.gl/edit-modes';
import { FeatureCollection } from '@nebula.gl/edit-modes';
import { GuideFeatureCollection } from '@nebula.gl/edit-modes';
import { ImmutableFeatureCollection } from '@nebula.gl/edit-modes';
import type { MapContextProps } from 'react-map-gl';
import { MeasureAngleMode } from '@nebula.gl/edit-modes';
import { MeasureAreaMode } from '@nebula.gl/edit-modes';
import { MeasureDistanceMode } from '@nebula.gl/edit-modes';
import { MjolnirEvent } from 'mjolnir.js';
import { ModeProps as ModeProps_2 } from '@nebula.gl/edit-modes';
import { Pick as Pick_2 } from '@nebula.gl/edit-modes';
import { PointerMoveEvent } from '@nebula.gl/edit-modes';
import type { Position as Position_2 } from '@nebula.gl/edit-modes';
import * as React_2 from 'react';
import { ScreenCoordinates } from '@nebula.gl/edit-modes';
import { StartDraggingEvent } from '@nebula.gl/edit-modes';
import { StopDraggingEvent } from '@nebula.gl/edit-modes';
import { Tooltip } from '@nebula.gl/edit-modes';
import type { WebMercatorViewport } from 'react-map-gl';
import { WebMercatorViewport as WebMercatorViewport_2 } from 'viewport-mercator-project';

export declare type BaseEvent = ClickEvent;

declare class BaseMode implements EditMode<FeatureCollection, GuideFeatureCollection> {
    _tentativeFeature: Feature | null | undefined;
    _editHandles: Feature[] | null | undefined;
    constructor();
    handlePan(event: ClickEvent, props: ModeProps<FeatureCollection>): void;
    handleClick(event: ClickEvent, props: ModeProps<FeatureCollection>): void;
    handleDblClick(event: ClickEvent, props: ModeProps<FeatureCollection>): void;
    handlePointerMove(event: PointerMoveEvent, props: ModeProps<FeatureCollection>): void;
    handleStartDragging(event: StartDraggingEvent, props: ModeProps<FeatureCollection>): void;
    handleStopDragging(event: StopDraggingEvent, props: ModeProps<FeatureCollection>): void;
    handleKeyUp(event: KeyboardEvent, props: ModeProps<FeatureCollection>): void;
    getGuides(props: ModeProps<FeatureCollection>): GuideFeatureCollection | null | undefined;
    getTooltips(props: ModeProps<FeatureCollection>): Tooltip[];
    handleDragging(event: DraggingEvent, props: ModeProps<FeatureCollection>): void;
    getTentativeFeature(): Feature;
    getEditHandles(): Feature[];
    setTentativeFeature(feature: Feature): void;
    getEditHandlesFromFeature(feature: Feature, featureIndex: number | null | undefined): any;
    getSelectedFeature(props: ModeProps<FeatureCollection>, featureIndex?: number | null): Feature;
}

export { DrawCircleByDiameterMode }

export { DrawCircleFromCenterMode }

export { DrawLineStringMode }

export { DrawPointMode }

export { DrawPolygonByDraggingMode }

export { DrawPolygonMode }

export { DrawRectangleMode }

export declare enum EDIT_TYPE {
    ADD_FEATURE = "addFeature",
    ADD_POSITION = "addPosition",
    REMOVE_POSITION = "removePosition",
    ADD_TENTATIVE_POSITION = "addTentativePosition",
    MOVE_POSITION = "movePosition",
    FINISH_MOVE_POSITION = "finishMovePosition"
}

declare function editHandleStyle({ feature, shape, index, state }: {
    feature: any;
    shape: any;
    index: any;
    state: any;
}): {};

export declare class EditingMode extends BaseMode {
    handleClick(event: ClickEvent, props: ModeProps<FeatureCollection>): void;
    handleStopDragging(event: StopDraggingEvent, props: ModeProps<FeatureCollection>): void;
    _handleDragging(event: PointerMoveEvent | StopDraggingEvent, props: ModeProps<FeatureCollection>): void;
    handlePointerMove(event: PointerMoveEvent, props: ModeProps<FeatureCollection>): void;
    _updateFeature(props: ModeProps<FeatureCollection>, type: string, options?: any): FeatureCollection;
    _getPointOnSegment(feature: Feature, picked: any, pickedMapCoords: Position_2): Position_2;
    _getCursorEditHandle(event: PointerMoveEvent, feature: Feature): {
        type: string;
        properties: {
            guideType: GUIDE_TYPE;
            shape: any;
            positionIndexes: number[];
            editHandleType: string;
        };
        geometry: {
            type: GEOJSON_TYPE;
            coordinates: Position_2;
        };
    };
    getGuides(props: ModeProps<FeatureCollection>): GuideFeatureCollection;
}

export declare class Editor extends ModeHandler {
    static defaultProps: {
        clickRadius: number;
        featureShape: string;
        editHandleShape: string;
        editHandleStyle: typeof editHandleStyle;
        featureStyle: typeof featureStyle;
        tooltipStyle: {
            fill: string;
            fontSize: number;
        };
        featuresDraggable: boolean;
        selectable: boolean;
        mode: any;
        features: any;
        onSelect: any;
        onUpdate: any;
        onUpdateCursor: () => void;
    };
    _getPathInScreenCoords(coordinates: any, type: GeoJsonType): any;
    _getEditHandleState: (editHandle: Feature, renderState?: string | null) => string;
    _getFeatureRenderState: (index: number, renderState?: RenderState | null) => RENDER_STATE;
    _getStyleProp: (styleProp: any, params: any) => any;
    _renderEditHandle: (editHandle: Feature, feature: Feature) => JSX.Element;
    _renderSegment: (featureIndex: Id, index: number, coordinates: number[], style: Record<string, any>) => JSX.Element;
    _renderSegments: (featureIndex: Id, coordinates: number[], style: Record<string, any>) => any[];
    _renderFill: (featureIndex: Id, coordinates: number[], style: Record<string, any>) => JSX.Element;
    _renderTentativeFeature: (feature: Feature, cursorEditHandle: Feature) => any[];
    _renderTooltips: (tooltips: Tooltip[]) => JSX.Element;
    _renderGuides: (guideFeatures: Feature[]) => JSX.Element;
    _renderPoint: (feature: Feature, index: number, path: string) => JSX.Element;
    _renderPath: (feature: Feature, index: number, path: string) => JSX.Element;
    _renderPolygon: (feature: Feature, index: number, path: string) => JSX.Element;
    _renderFeature: (feature: Feature, index: number) => JSX.Element;
    _renderCanvas: () => JSX.Element;
    _render: () => JSX.Element;
}

export declare type EditorProps = {
    mode?: Record<string, any>;
    style?: Record<string, any> | null | undefined;
    features?: Feature[] | null | undefined;
    selectedFeatureIndex?: number | null | undefined;
    selectable?: Boolean;
    clickRadius?: number;
    featureShape?: Function | string;
    editHandleShape?: Function | string;
    editHandleStyle?: Function | any;
    featureStyle?: Function | any;
    tooltipStyle?: Function | CSSProperties;
    featuresDraggable?: boolean | null | undefined;
    onUpdate?: Function;
    onSelect?: Function;
    onUpdateCursor?: Function;
    modeConfig?: any;
};

export declare type EditorState = {
    featureCollection: ImmutableFeatureCollection | null | undefined;
    selectedFeatureIndex: number | null | undefined;
    selectedEditHandleIndexes: number[];
    hovered: Pick_2 | null | undefined;
    lastPointerMoveEvent: PointerMoveEvent;
    isDragging: boolean;
    didDrag: boolean;
    pointerDownPicks: Pick_2[] | null | undefined;
    pointerDownScreenCoords: ScreenCoordinates | null | undefined;
    pointerDownMapCoords: Position | null | undefined;
};

export declare enum ELEMENT_TYPE {
    FEATURE = "feature",
    FILL = "fill",
    SEGMENT = "segment",
    EDIT_HANDLE = "editHandle",
    TOOLTIP = "tooltip"
}

declare function featureStyle({ feature, state }: {
    feature: any;
    state: any;
}): any;

export declare enum GEOJSON_TYPE {
    POINT = "Point",
    LINE_STRING = "LineString",
    POLYGON = "Polygon"
}

export declare type GeoJsonType = GEOJSON_TYPE;

declare enum GUIDE_TYPE {
    TENTATIVE = "tentative",
    EDIT_HANDLE = "editHandle",
    CURSOR_EDIT_HANDLE = "cursorEditHandle"
}

export declare type Id = string | number;

export { MeasureAngleMode }

export { MeasureAreaMode }

export { MeasureDistanceMode }

declare class ModeHandler extends React_2.PureComponent<EditorProps, EditorState> {
    static displayName: string;
    static defaultProps: {
        selectable: boolean;
        mode: any;
        features: any;
        onSelect: any;
        onUpdate: any;
        onUpdateCursor: () => void;
    };
    constructor(props: EditorProps);
    componentDidMount(): void;
    componentDidUpdate(prevProps: EditorProps): void;
    componentWillUnmount(): void;
    _events: Record<string, (evt: MjolnirEvent) => void>;
    _eventsRegistered: boolean;
    _modeHandler: any;
    _context: MapContextProps | null | undefined;
    _containerRef: HTMLElement | null | undefined;
    getFeatures: () => any;
    addFeatures: (features: Feature | Feature[]) => void;
    deleteFeatures: (featureIndexes: number | number[]) => void;
    deleteHandles: (featureIndex: number | undefined, handleIndexes: number[] | undefined) => FeatureCollection;
    getModeProps(): {
        data: any;
        selectedIndexes: number[];
        selectedEditHandleIndexes: number[];
        lastPointerMoveEvent: PointerMoveEvent;
        viewport: WebMercatorViewport;
        featuresDraggable: boolean;
        onEdit: (editAction: EditAction<any>) => void;
        onUpdateCursor: Function;
        modeConfig: any;
    };
    _getMemorizedFeatureCollection: (args: any) => any;
    _getFeatureCollection: () => any;
    _setupModeHandler: () => void;
    _clearEditingState: () => void;
    _getSelectedFeatureIndex: () => number;
    _onSelect: (selected: SelectAction) => void;
    _onEdit: (editAction: EditAction<any>) => void;
    _degregisterEvents: () => void;
    _registerEvents: () => void;
    _onEvent: (handler: Function, evt: MjolnirEvent, stopPropagation: boolean) => void;
    _onClick: (event: BaseEvent) => void;
    _onDblclick: (event: BaseEvent) => void;
    _onPointerMove: (event: BaseEvent) => void;
    _onPointerDown: (event: BaseEvent) => void;
    _onPointerUp: (event: BaseEvent) => void;
    _onPan: (event: BaseEvent) => void;
    project: (pt: [number, number]) => number[];
    unproject: (pt: [number, number]) => number[];
    _getEvent(evt: MjolnirEvent): {
        picks: {
            object: any;
            isGuide: boolean;
            type: string;
            index: string;
            featureIndex: string;
        }[];
        screenCoords: number[];
        mapCoords: number[];
        sourceEvent: MjolnirEvent;
    };
    _getHoverState: (event: BaseEvent) => {
        index: number;
        object?: any;
        isGuide?: boolean;
        featureIndex?: number;
        type?: string;
        screenCoords: ScreenCoordinates;
        mapCoords: Position_2;
    };
    _render(): JSX.Element;
    render(): JSX.Element;
}

export declare type ModeProps<TData> = ModeProps_2<TData> & {
    viewport: WebMercatorViewport_2 | null | undefined;
    featuresDraggable: boolean | null | undefined;
};

export declare type Position = [number, number] | [number, number, number];

export declare enum RENDER_STATE {
    INACTIVE = "INACTIVE",
    UNCOMMITTED = "UNCOMMITTED",
    SELECTED = "SELECTED",
    HOVERED = "HOVERED",
    CLOSING = "CLOSING"
}

declare enum RENDER_TYPE {
    POINT = "Point",
    LINE_STRING = "LineString",
    POLYGON = "Polygon",
    RECTANGLE = "Rectangle"
}

export declare type RenderState = RENDER_STATE;

export declare type RenderType = RENDER_TYPE;

export declare type SelectAction = {
    selectedFeature: Feature | null | undefined;
    selectedFeatureIndex?: number | null | undefined;
    selectedEditHandleIndex?: number | null | undefined;
    selectedEditHandleIndexes: number[];
    screenCoords: ScreenCoordinates | null | undefined;
    mapCoords: Position | null | undefined;
};

export declare enum SHAPE {
    POINT = "Point",
    LINE_STRING = "LineString",
    POLYGON = "Polygon",
    RECTANGLE = "Rectangle",
    CIRCLE = "Circle"
}

export { }
